@@ -24,21 +24,24 @@ Operation recorder interface and implementations.
 from __future__ import print_function, absolute_import
 from collections import namedtuple
 try:
-    from collections import OrderedDict  # pylint: disable=import-error
+    from collections import OrderedDict
 except ImportError:
     from ordereddict import OrderedDict  # pylint: disable=import-error
 from datetime import datetime, timedelta
 import logging
+
 import six
 import yaml
 from yaml.representer import RepresenterError
 
-from .cim_obj import CIMInstance, CIMInstanceName, CIMClass, CIMClassName, \
+from ._nocasedict import NocaseDict
+from ._cim_obj import CIMInstance, CIMInstanceName, CIMClass, CIMClassName, \
     CIMProperty, CIMMethod, CIMParameter, CIMQualifier, \
-    CIMQualifierDeclaration, NocaseDict, _ensure_unicode
-from .cim_types import CIMInt, CIMFloat, CIMDateTime
-from .exceptions import CIMError
+    CIMQualifierDeclaration
+from ._cim_types import CIMInt, CIMFloat, CIMDateTime
+from ._exceptions import CIMError
 from ._logging import LOGGER_API_CALLS_NAME, LOGGER_HTTP_NAME
+from ._utils import _ensure_unicode, _format
 
 if six.PY2:
     import codecs  # pylint: disable=wrong-import-order
@@ -49,8 +52,10 @@ __all__ = ['BaseOperationRecorder', 'TestClientRecorder',
            'OpArgs', 'OpResult', 'HttpRequest', 'HttpResponse']
 
 if six.PY2:
+    # pylint: disable=invalid-name, undefined-variable
     _Longint = long  # noqa: F821
 else:
+    # pylint: disable=invalid-name
     _Longint = int
 
 OpArgsTuple = namedtuple("OpArgsTuple", ["method", "args"])
@@ -61,7 +66,7 @@ def _represent_ordereddict(dump, tag, mapping, flow_style=None):
     This is needed for yaml.safe_dump() to support OrderedDict.
 
     Courtesy:
-    http://blog.elsdoerfer.name/2012/07/26/make-pyyaml-output-an-ordereddict/
+    https://blog.elsdoerfer.name/2012/07/26/make-pyyaml-output-an-ordereddict/
     """
     value = []
     node = yaml.MappingNode(tag, value, flow_style=flow_style)
@@ -75,10 +80,10 @@ def _represent_ordereddict(dump, tag, mapping, flow_style=None):
         node_value = dump.represent_data(item_value)
         if not (isinstance(node_key, yaml.ScalarNode) and
                 not node_key.style):
-                    best_style = False    # pylint: disable=bad-indentation
+            best_style = False    # pylint: disable=bad-indentation
         if not (isinstance(node_value, yaml.ScalarNode) and
                 not node_value.style):
-                    best_style = False    # pylint: disable=bad-indentation
+            best_style = False    # pylint: disable=bad-indentation
         value.append((node_key, node_value))
     if flow_style is None:
         if dump.default_flow_style is not None:
@@ -127,11 +132,12 @@ yaml.SafeLoader.add_constructor(CIMDATETIME_TAG, _cimdatetime_constructor)
 # Some monkey-patching for better diagnostics:
 def _represent_undefined(self, data):
     """Raises flag for objects that cannot be represented"""
-    raise RepresenterError("cannot represent an object: %s of type: %s; "
-                           "yaml_representers: %r, "
-                           "yaml_multi_representers: %r" %
-                           (data, type(data), self.yaml_representers.keys(),
-                            self.yaml_multi_representers.keys()))
+    raise RepresenterError(
+        _format("Cannot represent an object: {0!A} of type: {1}; "
+                "yaml_representers: {2!A}, "
+                "yaml_multi_representers: {3!A}",
+                data, type(data), self.yaml_representers.keys(),
+                self.yaml_multi_representers.keys()))
 
 
 yaml.SafeDumper.represent_undefined = _represent_undefined
@@ -139,11 +145,10 @@ yaml.SafeDumper.represent_undefined = _represent_undefined
 
 class OpArgs(OpArgsTuple):
     """
-    **Experimental:** *New in pywbem 0.9 as experimental.*
-
     A named tuple representing the name and input arguments of the invocation
-    of a :class:`~pywbem.WBEMConnection` method, with the following named fields
-    and attributes:
+    of a :class:`~pywbem.WBEMConnection` method.
+
+    **Experimental:** *New in pywbem 0.9 as experimental.*
 
     Attributes:
 
@@ -156,7 +161,11 @@ class OpArgs(OpArgsTuple):
     __slots__ = ()
 
     def __repr__(self):
-        return "OpArgs(method={s.method!r}, args={s.args!r})".format(s=self)
+        return _format(
+            "OpArgs("
+            "method={s.method!A}, "
+            "args={s.args!A})",
+            s=self)
 
 
 OpResultTuple = namedtuple("OpResultTuple", ["ret", "exc"])
@@ -164,11 +173,10 @@ OpResultTuple = namedtuple("OpResultTuple", ["ret", "exc"])
 
 class OpResult(OpResultTuple):
     """
-    **Experimental:** *New in pywbem 0.9 as experimental.*
-
     A named tuple representing the result of the invocation of a
-    :class:`~pywbem.WBEMConnection` method, with the following named fields
-    and attributes:
+    :class:`~pywbem.WBEMConnection` method.
+
+    **Experimental:** *New in pywbem 0.9 as experimental.*
 
     Attributes:
 
@@ -186,7 +194,11 @@ class OpResult(OpResultTuple):
     __slots__ = ()
 
     def __repr__(self):
-        return "OpResult(ret={s.ret!r}, exc={s.exc!r})".format(s=self)
+        return _format(
+            "OpResult("
+            "ret={s.ret!A}, "
+            "exc={s.exc!A})",
+            s=self)
 
 
 HttpRequestTuple = namedtuple("HttpRequestTuple",
@@ -196,10 +208,9 @@ HttpRequestTuple = namedtuple("HttpRequestTuple",
 
 class HttpRequest(HttpRequestTuple):
     """
-    **Experimental:** *New in pywbem 0.9 as experimental.*
+    A named tuple representing the HTTP request sent by the WBEM client.
 
-    A named tuple representing the HTTP request sent by the WBEM client, with
-    the following named fields and attributes:
+    **Experimental:** *New in pywbem 0.9 as experimental.*
 
     Attributes:
 
@@ -223,15 +234,19 @@ class HttpRequest(HttpRequestTuple):
 
       ~HttpRequest.payload (:term:`unicode string`):
         HTTP payload, i.e. the CIM-XML string.
-
     """
     __slots__ = ()
 
     def __repr__(self):
-        return "HttpRequest(version={s.version!r}, url={s.url!r}, " \
-               "target={s.target!r}, method={s.method!r}, " \
-               "headers={s.headers!r}, payload={s.payload!r})" \
-               .format(s=self)
+        return _format(
+            "HttpRequest("
+            "version={s.version!A}, "
+            "url={s.url!A}, "
+            "target={s.target!A}, "
+            "method={s.method!A}, "
+            "headers={s.headers!A}, "
+            "payload={s.payload!A})",
+            s=self)
 
 
 HttpResponseTuple = namedtuple("HttpResponseTuple",
@@ -241,10 +256,9 @@ HttpResponseTuple = namedtuple("HttpResponseTuple",
 
 class HttpResponse(HttpResponseTuple):
     """
-    **Experimental:** *New in pywbem 0.9 as experimental.*
+    A named tuple representing the HTTP response received by the WBEM client.
 
-    A named tuple representing the HTTP response received by the WBEM client,
-    with the following named fields and attributes:
+    **Experimental:** *New in pywbem 0.9 as experimental.*
 
     Attributes:
 
@@ -269,20 +283,25 @@ class HttpResponse(HttpResponseTuple):
     __slots__ = ()
 
     def __repr__(self):
-        return "HttpResponse(version={s.version!r}, status={s.status!r}, " \
-               "reason={s.reason!r}, headers={s.headers!r}, " \
-               "payload={s.payload!r})".format(s=self)
+        return _format(
+            "HttpResponse("
+            "version={s.version!A}, "
+            "status={s.status!A}, "
+            "reason={s.reason!A}, "
+            "headers={s.headers!A}, "
+            "payload={s.payload!A})",
+            s=self)
 
 
 class BaseOperationRecorder(object):
     # pylint: disable=too-many-instance-attributes
     """
-    **Experimental:** *New in pywbem 0.9 as experimental.*
-
     Abstract base class defining the interface to an operation recorder,
     that records the WBEM operations executed in a connection to a WBEM
     server.
 
+    **Experimental:** *New in pywbem 0.9 as experimental.*
+
     An operation recorder can be added to a connection via the
     :meth:`~pywbem.WBEMConnection.add_operation_recorder` method. The operation
     recorders of a connection can be retrieved via the
@@ -306,38 +325,39 @@ class BaseOperationRecorder(object):
 
     def enable(self):
         """
-        *New in pywbem 0.10.*
-
         Enable the recorder.
+
+        *New in pywbem 0.10.*
         """
         self._enabled = True
 
     def disable(self):
         """
-        *New in pywbem 0.10.*
-
         Disable the recorder.
+
+        *New in pywbem 0.10.*
         """
         self._enabled = False
 
     @property
     def enabled(self):
         """
-        *New in pywbem 0.10.*
-
         Indicate whether the recorder is enabled.
+
+        *New in pywbem 0.10.*
         """
         return self._enabled
 
     @staticmethod
     def open_file(filename, file_mode='w'):
         """
+        A static convenience function that performs the open of the recorder
+        file correctly for different versions of Python.
+
         *New in pywbem 0.10.*
 
-        A static convenience function that performs the open of the recorder
-        file correctly for different versions of Python.  This covers the
-        issue where the file should be opened in text mode but that is
-        done differently in Python 2 and Python 3.
+        This covers the issue where the file should be opened in text mode but
+        that is done differently in Python 2 and Python 3.
 
         The returned file-like object must be closed by the caller.
 
@@ -508,12 +528,12 @@ class BaseOperationRecorder(object):
 
 class LogOperationRecorder(BaseOperationRecorder):
     """
-    **Experimental:** *New in pywbem 0.11 and redesigned in pywbem 0.12 as
-    experimental.*
-
     A recorder that logs certain aspects of the WBEM operations driven by
     pywbem users to Python loggers.
 
+    **Experimental:** *New in pywbem 0.11 and redesigned in pywbem 0.12 as
+    experimental.*
+
     This recorder supports two Python loggers:
 
     * API logger (Python logger name: `'pywbem.api'`) - Logs API calls and
@@ -566,9 +586,9 @@ class LogOperationRecorder(BaseOperationRecorder):
         # build name for logger for this connection
         if conn_id:
             self.apilogger = logging.getLogger(
-                '%s.%s' % (LOGGER_API_CALLS_NAME, conn_id))
+                _format("{0}.{1}", LOGGER_API_CALLS_NAME, conn_id))
             self.httplogger = logging.getLogger(
-                '%s.%s' % (LOGGER_HTTP_NAME, conn_id))
+                _format("{0}.{1}", LOGGER_HTTP_NAME, conn_id))
         else:
             self.apilogger = logging.getLogger(LOGGER_API_CALLS_NAME)
             self.httplogger = logging.getLogger(LOGGER_HTTP_NAME)
@@ -666,13 +686,14 @@ class LogOperationRecorder(BaseOperationRecorder):
             """ format ret as repr while clipping it to max_len if
                 max_len is not None.
             """
-            # format the 'summary' and 'paths' detail_levels
-            if self.api_detail_level == 'summary':
+            # Format the 'summary' and 'paths' detail_levels
+            if self.api_detail_level == 'summary':  # pylint: disable=R1705
                 if isinstance(ret, list):
                     if ret:
                         ret_type = type(ret[0]).__name__ if ret else ""
-                        return 'list of %s; count=%s' % (ret_type, len(ret))
-                    return 'Empty'
+                        return _format("list of {0}; count={1}",
+                                       ret_type, len(ret))
+                    return "Empty"
 
                 ret_type = type(ret).__name__
                 if hasattr(ret, 'classname'):
@@ -681,25 +702,26 @@ class LogOperationRecorder(BaseOperationRecorder):
                     name = ret.name
                 else:
                     name = ""
-                return '%s %s' % (ret_type, name)
+                return _format("{0} {1}", ret_type, name)
 
             elif self.api_detail_level == 'paths':
                 if isinstance(ret, list):
                     if ret:
                         if hasattr(ret[0], 'path'):
-                            result_fmt = ', '.join([str(p.path) for p in ret])
+                            result_fmt = ', '.join(
+                                [_format("{0!A}", str(p.path)) for p in ret])
                         else:
-                            result_fmt = '%s' % ret
+                            result_fmt = _format("{0!A}", ret)
                     else:
                         result_fmt = ''
                 else:
                     if hasattr(ret, 'path'):
-                        result_fmt = '%s' % ret.path
+                        result_fmt = _format("{0!A}", str(ret.path))
                     else:
-                        result_fmt = '%s' % ret
+                        result_fmt = _format("{0!A}", ret)
 
             else:
-                result_fmt = '{0!r}'.format(ret)
+                result_fmt = _format("{0!A}", ret)
 
             if max_len and (len(result_fmt) > max_len):
                 result_fmt = result_fmt[:max_len] + '...'
@@ -710,7 +732,8 @@ class LogOperationRecorder(BaseOperationRecorder):
             if exc:  # format exception
                 # exceptions are always either all or reduced length
                 result = format_result(
-                    '%s(%s)' % (exc.__class__.__name__, exc), self.api_maxlen)
+                    _format("{0}({1})", exc.__class__.__name__, exc),
+                    self.api_maxlen)
 
                 return_type = 'Exception'
 
@@ -729,24 +752,27 @@ class LogOperationRecorder(BaseOperationRecorder):
                         data_str = 'paths'
 
                     try:    # test for query_result_class
-                        qrc = ', query_result_class=%s' % \
-                            ret.query_result_class
+                        qrc = _format(
+                            ", query_result_class={0}",
+                            ret.query_result_class)
                     except AttributeError:
                         pass
 
-                    result = '{0}(context={1}, eos={2}{3}, {4}={5})' \
-                        .format(type(ret).__name__, ret.context, ret.eos, qrc,
-                                data_str, format_result(rtn_data,
-                                                        self.api_maxlen))
+                    result = _format(
+                        "{0}(context={1}, eos={2}{3}, {4}={5})",
+                        type(ret).__name__, ret.context, ret.eos, qrc,
+                        data_str, format_result(rtn_data, self.api_maxlen))
 
                 # format enumerate response except not open/pull
                 elif isinstance(ret, list):
                     try:    # test for query_result_class
-                        qrc = ', query_result_class=%s' % ret.query_result_class
+                        qrc = _format(
+                            ", query_result_class={0}",
+                            ret.query_result_class)
                     except AttributeError:
                         pass
                     ret_fmtd = format_result(ret, self.api_maxlen)
-                    result = '%s%s' % (qrc, ret_fmtd)
+                    result = _format("{0}{1}", qrc, ret_fmtd)
 
                 # format single return object
                 else:
@@ -766,7 +792,8 @@ class LogOperationRecorder(BaseOperationRecorder):
             # if Auth header, mask data
             if 'Authorization' in headers:
                 authtype, cred = headers['Authorization'].split(' ')
-                headers['Authorization'] = '%s %s' % (authtype, 'X' * len(cred))
+                headers['Authorization'] = _format(
+                    "{0} {1}", authtype, 'X' * len(cred))
 
             header_str = ' '.join('{0}:{1!r}'.format(k, v)
                                   for k, v in headers.items())
@@ -778,8 +805,10 @@ class LogOperationRecorder(BaseOperationRecorder):
                 upayload = payload
             if self.http_maxlen and (len(payload) > self.http_maxlen):
                 upayload = upayload[:self.http_maxlen] + '...'
-
-            self.httplogger.debug('Request:%s %s %s %s %s %s\n    %s',
+            upayload = repr(upayload)
+            if upayload.startswith("u'"):
+                upayload = upayload[1:]
+            self.httplogger.debug('Request:%s %s %s %s %s %s %s',
                                   conn_id, method, target, version, url,
                                   header_str, upayload)
 
@@ -815,8 +844,11 @@ class LogOperationRecorder(BaseOperationRecorder):
                             '...')
             else:
                 upayload = _ensure_unicode(payload)
+            upayload = repr(upayload)
+            if upayload.startswith("u'"):
+                upayload = upayload[1:]
 
-            self.httplogger.debug('Response:%s %s:%s %s %s\n    %s',
+            self.httplogger.debug('Response:%s %s:%s %s %s %s',
                                   self._http_response_conn_id,
                                   self._http_response_status,
                                   self._http_response_reason,
@@ -837,10 +869,12 @@ class LogOperationRecorder(BaseOperationRecorder):
 
 class TestClientRecorder(BaseOperationRecorder):
     """
+    An operation recorder that generates test cases for each recorded
+    operation.
+
     **Experimental:** *New in pywbem 0.9 as experimental.*
 
-    An operation recorder that generates test cases for each recorded
-    operation. The test cases are in the YAML format suitable for the
+    The test cases are in the YAML format suitable for the
     `test_client` unit test module of the pywbem project.
     """
 
@@ -990,7 +1024,7 @@ class TestClientRecorder(BaseOperationRecorder):
 
         # namedtuple is subclass of tuple so it is instance of tuple but
         # not type tuple. Cvt to dictionary and cvt dict to yaml.
-        # pylint: disable=unidiomatic-typecheck
+        # pylint: disable=unidiomatic-typecheck, no-else-return
         if isinstance(obj, tuple) and type(obj) is not tuple:
             ret_dict = obj._asdict()
             return self.toyaml(ret_dict)
@@ -1125,5 +1159,6 @@ class TestClientRecorder(BaseOperationRecorder):
             ret_dict['translatable'] = self.toyaml(obj.translatable)
             return ret_dict
         else:
-            raise TypeError("Invalid type in TestClientRecorder.toyaml(): "
-                            "%s %s" % (obj.__class__.__name__, type(obj)))
+            raise TypeError(
+                _format("Invalid type in TestClientRecorder.toyaml(): {0} {1}",
+                        obj.__class__.__name__, type(obj)))
