@@ -146,10 +146,11 @@ import uuid
 import six
 
 from ._server import WBEMServer
-from .cim_obj import CIMInstance, CIMInstanceName
-from .cim_http import parse_url
-from .cim_constants import CIM_ERR_FAILED
-from .exceptions import CIMError
+from ._cim_obj import CIMInstance, CIMInstanceName
+from ._cim_http import parse_url
+from ._cim_constants import CIM_ERR_FAILED
+from ._exceptions import CIMError
+from ._utils import _format
 
 # CIM model classnames for subscription components
 SUBSCRIPTION_CLASSNAME = 'CIM_IndicationSubscription'
@@ -212,24 +213,40 @@ class WBEMSubscriptionManager(object):
         if subscription_manager_id is None:
             raise ValueError("Subscription manager ID must not be None")
         if not isinstance(subscription_manager_id, six.string_types):
-            raise TypeError("Invalid type for subscription manager ID: %r" %
-                            subscription_manager_id)
+            raise TypeError(
+                _format("Invalid type for subscription manager ID: {0!A}",
+                        subscription_manager_id))
         if ':' in subscription_manager_id:
-            raise ValueError("Subscription manager ID contains ':': %s" %
-                             subscription_manager_id)
+            raise ValueError(
+                _format("Subscription manager ID contains ':': {0!A}",
+                        subscription_manager_id))
         self._subscription_manager_id = subscription_manager_id
 
+    def __str__(self):
+        """
+        Return a representation of the :class:`~pywbem.WBEMSubscriptionManager`
+        object with a subset of its attributes.
+        """
+        return _format(
+            "WBEMSubscriptionManager("
+            "_subscription_manager_id={s._subscription_manager_id!A}, "
+            "_servers={s._servers}, "
+            "...)",
+            s=self)
+
     def __repr__(self):
         """
         Return a representation of the :class:`~pywbem.WBEMSubscriptionManager`
         object with all attributes, that is suitable for debugging.
         """
-        return "%s(_subscription_manager_id=%r, _servers=%r, " \
-               "_owned_subscriptions=%r, _owned_filters=%r, " \
-               "_owned_destinations=%r)" % \
-               (self.__class__.__name__, self._subscription_manager_id,
-                self._servers, self._owned_subscriptions,
-                self._owned_filters, self._owned_destinations)
+        return _format(
+            "WBEMSubscriptionManager("
+            "_subscription_manager_id={s._subscription_manager_id!A}, "
+            "_servers={s._servers!A}, "
+            "_owned_subscriptions={s._owned_subscriptions!A}, "
+            "_owned_filters={s._owned_filters!A}, "
+            "_owned_destinations={s._owned_destinations!A})",
+            s=self)
 
     def __enter__(self):
         """
@@ -273,8 +290,9 @@ class WBEMSubscriptionManager(object):
         """
 
         if server_id not in self._servers:
-            raise ValueError('WBEM server %s not known by subscription '
-                             'manager' % server_id)
+            raise ValueError(
+                _format("WBEM server {0!A} not known by subscription manager",
+                        server_id))
 
         return self._servers[server_id]
 
@@ -318,8 +336,9 @@ class WBEMSubscriptionManager(object):
                             "WBEMServer object")
         server_id = server.url
         if server_id in self._servers:
-            raise ValueError("WBEM server already known by listener: %s" %
-                             server_id)
+            raise ValueError(
+                _format("WBEM server already known by listener: {0!A}",
+                        server_id))
 
         # Create dictionary entries for this server
         self._servers[server_id] = server
@@ -333,8 +352,8 @@ class WBEMSubscriptionManager(object):
         this_host = getfqdn()
 
         dest_name_pattern = re.compile(
-            r'^pywbemdestination:owned:%s:%s:[^:]*$' %
-            (this_host, self._subscription_manager_id))
+            _format(r'^pywbemdestination:owned:{0}:{1}:[^:]*$',
+                    this_host, self._subscription_manager_id))
         dest_insts = server.conn.EnumerateInstances(
             DESTINATION_CLASSNAME, namespace=server.interop_ns)
         for inst in dest_insts:
@@ -343,8 +362,8 @@ class WBEMSubscriptionManager(object):
                 self._owned_destinations[server_id].append(inst)
 
         filter_name_pattern = re.compile(
-            r'^pywbemfilter:owned:%s:%s:[^:]*:[^:]*$' %
-            (this_host, self._subscription_manager_id))
+            _format(r'^pywbemfilter:owned:{0}:{1}:[^:]*:[^:]*$',
+                    this_host, self._subscription_manager_id))
         filter_insts = server.conn.EnumerateInstances(
             FILTER_CLASSNAME, namespace=server.interop_ns)
         for inst in filter_insts:
@@ -624,6 +643,7 @@ class WBEMSubscriptionManager(object):
 
         # Validate server_id
         server = self._get_server(server_id)
+        conn_id = server.conn.conn_id if server.conn is not None else None
 
         # If list, recursively call this function with each list item.
         if isinstance(destination_paths, list):
@@ -640,9 +660,10 @@ class WBEMSubscriptionManager(object):
         if ref_paths:
             # DSP1054 1.2 defines that this CIM error is raised by the server
             # in that case, so we simulate that behavior on the client side.
-            raise CIMError(CIM_ERR_FAILED,
-                           "The listener destination is referenced by "
-                           "subscriptions.")
+            raise CIMError(
+                CIM_ERR_FAILED,
+                "The listener destination is referenced by subscriptions.",
+                conn_id=conn_id)
 
         server.conn.DeleteInstance(dest_path)
 
@@ -769,9 +790,11 @@ class WBEMSubscriptionManager(object):
                                  "specified, but only one of them must be "
                                  "specified")
             if not isinstance(filter_id, six.string_types):
-                raise TypeError("Invalid type for filter ID: %r" % filter_id)
+                raise TypeError(
+                    _format("Invalid type for filter ID: {0!A}", filter_id))
             if ':' in filter_id:
-                raise ValueError("Filter ID contains ':': %s" % filter_id)
+                raise ValueError(
+                    _format("Filter ID contains ':': {0!A}", filter_id))
 
         filter_inst = self._create_filter(server_id, source_namespace, query,
                                           query_language, owned, filter_id,
@@ -867,6 +890,7 @@ class WBEMSubscriptionManager(object):
 
         # Validate server_id
         server = self._get_server(server_id)
+        conn_id = server.conn.conn_id if server.conn is not None else None
 
         # Verify referencing subscriptions.
         ref_paths = server.conn.ReferenceNames(
@@ -874,9 +898,10 @@ class WBEMSubscriptionManager(object):
         if ref_paths:
             # DSP1054 1.2 defines that this CIM error is raised by the server
             # in that case, so we simulate that behavior on the client side.
-            raise CIMError(CIM_ERR_FAILED,
-                           "The indication filter is referenced by "
-                           "subscriptions.")
+            raise CIMError(
+                CIM_ERR_FAILED,
+                "The indication filter is referenced by subscriptions.",
+                conn_id=conn_id)
 
         server.conn.DeleteInstance(filter_path)
 
@@ -982,11 +1007,13 @@ class WBEMSubscriptionManager(object):
         # filter or on an owned destination.
         if not owned:
             if filter_path in owned_filter_paths:
-                raise ValueError("Permanent subscription cannot be created on "
-                                 "owned filter: %s" % filter_path)
+                raise ValueError(
+                    _format("Permanent subscription cannot be created on "
+                            "owned filter: {0!A}", filter_path))
             if dest_path in owned_destination_paths:
-                raise ValueError("Permanent subscription cannot be created on "
-                                 "owned listener destination: %s" % dest_path)
+                raise ValueError(
+                    _format("Permanent subscription cannot be created on "
+                            "owned listener destination: {0!A}", dest_path))
 
         sub_inst = self._create_subscription(server_id, dest_path, filter_path,
                                              owned)
@@ -1140,7 +1167,7 @@ class WBEMSubscriptionManager(object):
         # validate the URL by reconstructing it. Do not allow defaults
         host, port, ssl = parse_url(dest_url, allow_defaults=False)
         schema = 'https' if ssl else 'http'
-        listener_url = '{}://{}:{}'.format(schema, host, port)
+        listener_url = '{0}://{1}:{2}'.format(schema, host, port)
 
         this_host = getfqdn()
         ownership = "owned" if owned else "permanent"
@@ -1153,12 +1180,12 @@ class WBEMSubscriptionManager(object):
         dest_inst['CreationClassName'] = DESTINATION_CLASSNAME
         dest_inst['SystemCreationClassName'] = SYSTEM_CREATION_CLASSNAME
         dest_inst['SystemName'] = this_host
-        dest_inst['Name'] = 'pywbemdestination:%s:%s:%s' % \
-                            (ownership, self._subscription_manager_id,
-                             uuid.uuid4())
+        dest_inst['Name'] = _format(
+            'pywbemdestination:{0}:{1}:{2}',
+            ownership, self._subscription_manager_id, uuid.uuid4())
         dest_inst['Destination'] = listener_url
 
-        if owned:  # pylint: disable=no-else-return
+        if owned:
             for i, inst in enumerate(self._owned_destinations[server_id]):
                 if inst.path == dest_path:
                     # It already exists, now check its properties
@@ -1170,12 +1197,12 @@ class WBEMSubscriptionManager(object):
             dest_path = server.conn.CreateInstance(dest_inst)
             dest_inst = server.conn.GetInstance(dest_path)
             self._owned_destinations[server_id].append(dest_inst)
-            return dest_inst
         else:
             # Responsibility to ensure it does not exist yet is with the user
             dest_path = server.conn.CreateInstance(dest_inst)
             dest_inst = server.conn.GetInstance(dest_path)
-            return dest_inst
+
+        return dest_inst
 
     def _create_filter(self, server_id, source_namespace, query,
                        query_language, owned, filter_id, name):
@@ -1242,16 +1269,17 @@ class WBEMSubscriptionManager(object):
         filter_inst['SystemCreationClassName'] = SYSTEM_CREATION_CLASSNAME
         filter_inst['SystemName'] = this_host
         if filter_id:
-            filter_inst['Name'] = 'pywbemfilter:%s:%s:%s:%s' % \
-                (ownership, self._subscription_manager_id, filter_id,
-                 uuid.uuid4())
+            filter_inst['Name'] = _format(
+                'pywbemfilter:{0}:{1}:{2}:{3}',
+                ownership, self._subscription_manager_id, filter_id,
+                uuid.uuid4())
         if name:
             filter_inst['Name'] = name
         filter_inst['SourceNamespace'] = source_namespace
         filter_inst['Query'] = query
         filter_inst['QueryLanguage'] = query_language
 
-        if owned:  # pylint: disable=no-else-return
+        if owned:
             for i, inst in enumerate(self._owned_filters[server_id]):
                 if inst.path == filter_path:
                     # It already exists, now check its properties
@@ -1263,12 +1291,12 @@ class WBEMSubscriptionManager(object):
             filter_path = server.conn.CreateInstance(filter_inst)
             filter_inst = server.conn.GetInstance(filter_path)
             self._owned_filters[server_id].append(filter_inst)
-            return filter_inst
         else:
             # Responsibility to ensure it does not exist yet is with the user
             filter_path = server.conn.CreateInstance(filter_inst)
             filter_inst = server.conn.GetInstance(filter_path)
-            return filter_inst
+
+        return filter_inst
 
     def _create_subscription(self, server_id, dest_path, filter_path, owned):
         """
@@ -1317,7 +1345,7 @@ class WBEMSubscriptionManager(object):
         sub_inst['Filter'] = filter_path
         sub_inst['Handler'] = dest_path
 
-        if owned:  # pylint: disable=no-else-return
+        if owned:
             for inst in self._owned_subscriptions[server_id]:
                 if inst.path == sub_path:
                     # It does not have any properties besides its keys,
@@ -1326,9 +1354,9 @@ class WBEMSubscriptionManager(object):
             sub_path = server.conn.CreateInstance(sub_inst)
             sub_inst = server.conn.GetInstance(sub_path)
             self._owned_subscriptions[server_id].append(sub_inst)
-            return sub_inst
         else:
             # Responsibility to ensure it does not exist yet is with the user
             sub_path = server.conn.CreateInstance(sub_inst)
             sub_inst = server.conn.GetInstance(sub_path)
-            return sub_inst
+
+        return sub_inst
