@@ -15,19 +15,25 @@
 #
 
 """
-The :class:`~pywbem.ValueMapping` class supports translating the values of an
-integer-typed CIM element (e.g. property, method, or parameter) that is
-qualified with the `ValueMap` and `Values` qualifiers, to the corresponding
-value of the `Values` qualifier.
+The :class:`~pywbem.ValueMapping` class supports translating between the values
+of an integer-typed CIM element (e.g. property, method, or parameter) that is
+qualified with the `ValueMap` and `Values` qualifiers, and the corresponding
+values of the `Values` qualifier, in both directions.
 
 This class supports value ranges (e.g. ``"4..6"``) and the unclaimed marker
 (``".."``).
 """
 
 import re
+try:
+    from collections import OrderedDict
+except ImportError:
+    from ordereddict import OrderedDict
 import six
 
-from .cim_types import CIMInt, type_from_name
+from ._cim_types import CIMInt, type_from_name
+from ._cim_obj import CIMProperty, CIMMethod, CIMParameter
+from ._utils import _format, _integerValue_to_int
 
 __all__ = ['ValueMapping']
 
@@ -36,17 +42,18 @@ class ValueMapping(object):
     """
     *New in pywbem 0.9 as experimental and finalized in 0.10.*
 
-    A utility class that supports translating the values of an integer-typed
-    CIM element (property, method, parameter) that is qualified with the
-    `ValueMap` and `Values` qualifiers, to the corresponding value of the
-    `Values` qualifier.
+    A utility class that supports translating between the values of an
+    integer-typed CIM element (property, method, parameter) that is qualified
+    with the `ValueMap` and `Values` qualifiers, and the corresponding values
+    of the `Values` qualifier, in both directions.
 
     This is done by retrieving the CIM class definition defining the CIM
     element in question, and by inspecting its `ValueMap` and `Values`
     qualifiers.
 
-    The translation of the values is performed by the
-    :meth:`~pywbem.ValueMapping.tovalues` method.
+    The translation is performed by the
+    :meth:`~pywbem.ValueMapping.tovalues` and
+    :meth:`~pywbem.ValueMapping.tobinary` methods.
 
     Instances of this class must be created through one of the factory class
     methods: :meth:`~pywbem.ValueMapping.for_property`,
@@ -56,6 +63,10 @@ class ValueMapping(object):
     Value ranges (``"2..4"``) and the indicator for unclaimed values (``".."``)
     in the `ValueMap` qualifier are supported.
 
+    All representations of the integer values in the `ValueMap` qualifier are
+    supported (decimal, binary, octal, hexadecimal), consistent with the
+    definition of the `ValueMap` qualifier in :term:`DSP0004`.
+
     Example:
 
       Given the following definition of a property in MOF:
@@ -72,8 +83,10 @@ class ValueMapping(object):
           };
 
       Assuming this class exists in a WBEM server, the following code will
-      create a value mapping for this property and will print a few property
-      values and their corresponding `Values` strings::
+      get the class from the server, create a value mapping for this property,
+      and look up the `Values` strings that correspond to binary property
+      values. This is useful when preparing binary property values for human
+      consumption::
 
           namespace = 'root/cimv2'
           conn = pywbem.WBEMConnection(...)  # WBEM server
@@ -81,27 +94,92 @@ class ValueMapping(object):
           myprop_vm = pywbem.ValueMapping.for_property(
               conn, namespace, 'CIM_Foo', 'MyProp')
 
-          print("value: Values")
-          for value in range(0, 12):
-              print("%5s: %r" % (value, myprop_vm.tovalues(value))
+          print("Binary value: Values string")
+          for bin_value in range(0, 12):
+              values_str = myprop_vm.tovalues(bin_value)
+              print("{0:12}: {1!r}".format(bin_value, values_str))
 
       Resulting output:
 
       .. code-block:: text
 
-          value: Values
-              0: 'zero'
-              1: 'unclaimed'
-              2: 'two-four'
-              3: 'two-four'
-              4: 'two-four'
-              5: 'five-six'
-              6: 'five-six'
-              7: 'seven-eight'
-              8: 'seven-eight'
-              9: 'nine'
-             10: 'unclaimed'
-             11: 'unclaimed'
+          Binary value: Values string
+                     0: 'zero'
+                     1: 'unclaimed'
+                     2: 'two-four'
+                     3: 'two-four'
+                     4: 'two-four'
+                     5: 'five-six'
+                     6: 'five-six'
+                     7: 'seven-eight'
+                     8: 'seven-eight'
+                     9: 'nine'
+                    10: 'unclaimed'
+                    11: 'unclaimed'
+
+      Translating in the other direction is also of interest, for example when
+      processing values that are provided by humans in terms of the `Values`
+      strings, or when using the pywbem mock support and the test cases specify
+      property values for CIM instances in terms of the more human-readable
+      `Values` strings.
+
+      Again, assuming the class shown above exists in a WBEM server, the
+      following code will get the class from the server, create a value
+      mapping for this property, and look up the binary property values from
+      the `Values` strings::
+
+          namespace = 'root/cimv2'
+          conn = pywbem.WBEMConnection(...)  # WBEM server
+
+          myprop_vm = pywbem.ValueMapping.for_property(
+              conn, namespace, 'CIM_Foo', 'MyProp')
+
+          values_strs = ["zero", "two-four", "five-six", "seven-eight", "nine",
+                         "unclaimed"]
+
+          print("Values string: Binary value")
+          for values_str in values_strs:
+              bin_value = myprop_vm.tobinary(values_str)
+              print("{0:12}: {1!r}".format(values_str, bin_value))
+
+      Resulting output:
+
+      .. code-block:: text
+
+          Values string: Binary value
+                 'zero': 0
+             'two-four': (2, 4)
+             'five-six': (5, 6)
+          'seven-eight': (7, 8)
+                 'nine': 9
+            'unclaimed': None
+
+      Iterating through the pairs of `ValueMap` and `Values` entries is
+      also possible. Assuming the class shown above exists in a WBEM server, the
+      following code will get the class from the server, and iterate through the
+      value mapping::
+
+          namespace = 'root/cimv2'
+          conn = pywbem.WBEMConnection(...)  # WBEM server
+
+          myprop_vm = pywbem.ValueMapping.for_property(
+              conn, namespace, 'CIM_Foo', 'MyProp')
+
+          print("Values string: Binary value")
+          for bin_value, values_str in myprop_vm.items():
+              print("{0:12}: {1!r}".format(values_str, bin_value))
+
+      Resulting output:
+
+      .. code-block:: text
+
+          Values string: Binary value
+                 'zero': 0
+             'two-four': (2, 4)
+             'five-six': (5, 6)
+          'seven-eight': (7, 8)
+                 'nine': 9
+            'unclaimed': None
     """
 
     def __init__(self):
@@ -115,9 +193,13 @@ class ValueMapping(object):
 
         self._element_obj = None
 
-        self._single_dict = {}  # for single values; elem_val: values_str)
-        self._range_tuple_list = []  # for value ranges; tuple(lo,hi,values_str)
-        self._unclaimed = None  # value of the unclaimed indicator '..'
+        # Attributes for converting binary values to Values strings:
+        self._b2v_single_dict = {}  # for single values; bin: values
+        self._b2v_range_tuple_list = []  # for value ranges; tuple(lo,hi,values)
+        self._b2v_unclaimed = None  # value of the unclaimed indicator '..'
+
+        # Attributes for converting Values strings to binary values:
+        self._v2b_dict = {}  # values: bin (int or tuple)
 
     @classmethod
     def for_property(cls, server, namespace, classname, propname):
@@ -155,9 +237,11 @@ class ValueMapping(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ValueError: No `Values` qualifier defined.
-            TypeError: The property is not integer-typed.
-            KeyError: The property does not exist in the class.
+            KeyError: The CIM property does not exist in the CIM class.
+            TypeError: The CIM property is not integer-typed.
+            ValueError: No `Values` qualifier defined on the CIM property.
+            ValueError: Invalid integer representation in `ValueMap` qualifier
+              defined on the CIM property.
         """  # noqa: E501
 
         conn = server
@@ -171,7 +255,12 @@ class ValueMapping(object):
                               namespace=namespace,
                               LocalOnly=False,
                               IncludeQualifiers=True)
-        property_obj = class_obj.properties[propname]
+        try:
+            property_obj = class_obj.properties[propname]
+        except KeyError:
+            raise KeyError(
+                _format("Class {0!A} (in {1!A}) does not have a property "
+                        "{2!A}", classname, namespace, propname))
 
         new_vm = cls._create_for_element(property_obj, conn, namespace,
                                          classname, propname=propname)
@@ -213,9 +302,11 @@ class ValueMapping(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ValueError: No `Values` qualifier defined.
-            TypeError: The method is not integer-typed.
-            KeyError: The method does not exist in the class.
+            KeyError: The CIM method does not exist in the CIM class.
+            TypeError: The CIM method is not integer-typed.
+            ValueError: No `Values` qualifier defined on the CIM method.
+            ValueError: Invalid integer representation in `ValueMap` qualifier
+              defined on the CIM method.
         """  # noqa: E501
 
         conn = server
@@ -229,7 +320,12 @@ class ValueMapping(object):
                               namespace=namespace,
                               LocalOnly=False,
                               IncludeQualifiers=True)
-        method_obj = class_obj.methods[methodname]
+        try:
+            method_obj = class_obj.methods[methodname]
+        except KeyError:
+            raise KeyError(
+                _format("Class {0!A} (in {1!A}) does not have a method {2!A}",
+                        classname, namespace, methodname))
 
         new_vm = cls._create_for_element(method_obj, conn, namespace,
                                          classname, methodname=methodname)
@@ -275,10 +371,12 @@ class ValueMapping(object):
         Raises:
 
             Exceptions raised by :class:`~pywbem.WBEMConnection`.
-            ValueError: No `Values` qualifier defined.
-            TypeError: The parameter is not integer-typed.
-            KeyError: The method does not exist in the class, or the
-              parameter does not exist in the method.
+            KeyError: The CIM method does not exist in the CIM class.
+            KeyError: The CIM parameter does not exist in the CIM method.
+            TypeError: The CIM parameter is not integer-typed.
+            ValueError: No `Values` qualifier defined on the CIM parameter.
+            ValueError: Invalid integer representation in `ValueMap` qualifier
+              defined on the CIM parameter.
         """  # noqa: E501
 
         conn = server
@@ -292,8 +390,19 @@ class ValueMapping(object):
                               namespace=namespace,
                               LocalOnly=False,
                               IncludeQualifiers=True)
-        method_obj = class_obj.methods[methodname]
-        parameter_obj = method_obj.parameters[parametername]
+        try:
+            method_obj = class_obj.methods[methodname]
+        except KeyError:
+            raise KeyError(
+                _format("Class {0!A} (in {1!A}) does not have a method {2!A}",
+                        classname, namespace, methodname))
+        try:
+            parameter_obj = method_obj.parameters[parametername]
+        except KeyError:
+            raise KeyError(
+                _format("Method {0!A} in class {1!A} (in {2!A}) does not have "
+                        "a parameter {3!A}", methodname, classname, namespace,
+                        parametername))
 
         new_vm = cls._create_for_element(parameter_obj, conn, namespace,
                                          classname, methodname=methodname,
@@ -301,10 +410,10 @@ class ValueMapping(object):
 
         return new_vm
 
-    @classmethod
-    def _values_tuple(cls, i, valuemap_list, values_list, cimtype):
+    def _values_tuple(self, i, valuemap_list, values_list, cimtype):
         """
-        Return a tuple for the value range at position i, with these items:
+        Return a tuple for the value range or unclaimed marker at position i,
+        with these items:
 
         * lo - low value of the range
         * hi - high value of the range (can be equal to lo)
@@ -322,38 +431,49 @@ class ValueMapping(object):
 
         Raises:
 
-            ValueError: Invalid `ValueMap` entry.
+            ValueError: Invalid integer representation in `ValueMap` qualifier
+              defined on the CIM element.
         """
         values_str = values_list[i]
         valuemap_str = valuemap_list[i]
-        try:
-            valuemap_int = int(valuemap_str)
+        m = re.match(r'^(.*)\.\.(.*)$', valuemap_str)
+        if m is None:
+            valuemap_int = self._to_int(valuemap_str)
             return (valuemap_int, valuemap_int, values_str)
-        except ValueError:
-            m = re.match(r'^([+-]?[0-9]*)\.\.([+-]?[0-9]*)$', valuemap_str)
-            if m is None:
-                raise ValueError("Invalid ValueMap entry: %r" % valuemap_str)
-            lo = m.group(1)
-            if lo == '':
-                if i == 0:
-                    lo = cimtype.minvalue
-                else:
-                    _, previous_hi, _ = cls._values_tuple(
-                        i - 1, valuemap_list, values_list, cimtype)
-                    lo = previous_hi + 1
+
+        # match found
+        lo = m.group(1)
+        if lo == '':
+            if i == 0:
+                lo = cimtype.minvalue
             else:
-                lo = int(lo)
-            hi = m.group(2)
-            if hi == '':
-                if i == len(valuemap_list) - 1:
-                    hi = cimtype.maxvalue
-                else:
-                    next_lo, _, _ = cls._values_tuple(
-                        i + 1, valuemap_list, values_list, cimtype)
-                    hi = next_lo - 1
+                _, previous_hi, _ = self._values_tuple(
+                    i - 1, valuemap_list, values_list, cimtype)
+                lo = previous_hi + 1
+        else:
+            lo = self._to_int(lo)
+
+        hi = m.group(2)
+        if hi == '':
+            if i == len(valuemap_list) - 1:
+                hi = cimtype.maxvalue
             else:
-                hi = int(hi)
-            return (lo, hi, values_str)
+                next_lo, _, _ = self._values_tuple(
+                    i + 1, valuemap_list, values_list, cimtype)
+                hi = next_lo - 1
+        else:
+            hi = self._to_int(hi)
+        return (lo, hi, values_str)
+
+    def _to_int(self, val_str):
+        """Conver val_str to an integer or raise ValueError"""
+        val = _integerValue_to_int(val_str)
+        if val is None:
+            raise ValueError(
+                _format("The value-mapped {0} has an invalid integer "
+                        "representation in a ValueMap entry: {1!A}",
+                        self._element_str(), val_str))
+        return val
 
     @classmethod
     def _create_for_element(cls, element_obj, conn, namespace, classname,
@@ -400,24 +520,15 @@ class ValueMapping(object):
 
         Raises:
 
-            ValueError: No `Values` qualifier defined.
-            ValueError: Invalid `ValueMap` entry.
             TypeError: The CIM element is not integer-typed.
+            ValueError: No `Values` qualifier defined on the CIM element.
+            ValueError: Invalid integer representation in `ValueMap` qualifier
+              defined on the CIM element.
         """  # noqa: E501
+        # pylint: enable=line-too-long
 
         # pylint: disable=protected-access
 
-        try:
-            typename = element_obj.type  # Property, Parameter
-        except AttributeError:
-            typename = element_obj.return_type  # Method
-
-        cimtype = type_from_name(typename)
-
-        if not issubclass(cimtype, CIMInt):
-            raise TypeError("The CIM element is not integer-typed: %s" %
-                            typename)
-
         vm = ValueMapping()
         vm._element_obj = element_obj
         vm._conn = conn
@@ -427,53 +538,96 @@ class ValueMapping(object):
         vm._methodname = methodname
         vm._parametername = parametername
 
+        try:
+            typename = element_obj.type  # Property, Parameter
+        except AttributeError:
+            typename = element_obj.return_type  # Method
+
+        cimtype = type_from_name(typename)
+
+        if not issubclass(cimtype, CIMInt):
+            raise TypeError(
+                _format("The value-mapped {0} is not integer-typed, but "
+                        "has CIM type: {1}", vm._element_str(), typename))
+
         values_qual = element_obj.qualifiers.get('Values', None)
         if values_qual is None:
             # DSP0004 defines no default for a missing Values qualifier
-            raise ValueError("No Values qualifier defined")
+            raise ValueError(
+                _format("The value-mapped {0} has no Values qualifier "
+                        "defined", vm._element_str()))
         values_list = values_qual.value
 
         valuemap_qual = element_obj.qualifiers.get('ValueMap', None)
         if valuemap_qual is None:
             # DSP0004 defines a default of consecutive index numbers
-            vm._single_dict = dict(zip(range(0, len(values_list)), values_list))
-            vm._range_tuple_list = []
-            vm._unclaimed = None
+            vm._b2v_single_dict = dict(zip(range(0, len(values_list)),
+                                           values_list))
+            vm._b2v_range_tuple_list = []
+            vm._b2v_unclaimed = None
+            vm._v2b_dict = OrderedDict(zip(values_list,
+                                           range(0, len(values_list))))
         else:
-            vm._single_dict = {}
-            vm._range_tuple_list = []
-            vm._unclaimed = None
+            vm._b2v_single_dict = {}
+            vm._b2v_range_tuple_list = []
+            vm._b2v_unclaimed = None
+            vm._v2b_dict = OrderedDict()
             valuemap_list = valuemap_qual.value
             for i, valuemap_str in enumerate(valuemap_list):
                 values_str = values_list[i]
                 if valuemap_str == '..':
-                    vm._unclaimed = values_str
+                    vm._b2v_unclaimed = values_str
+                    vm._v2b_dict[values_str] = None
                 else:
-                    lo, hi, values_str = cls._values_tuple(
+                    lo, hi, values_str = vm._values_tuple(
                         i, valuemap_list, values_list, cimtype)
                     if lo == hi:
                         # single value
-                        vm._single_dict[lo] = values_str
+                        vm._b2v_single_dict[lo] = values_str
+                        vm._v2b_dict[values_str] = lo
                     else:
                         # value range
-                        vm._range_tuple_list.append((lo, hi, values_str))
+                        vm._b2v_range_tuple_list.append((lo, hi, values_str))
+                        vm._v2b_dict[values_str] = (lo, hi)
 
-        # pylint: enable=protected-access
         return vm
 
+    def _element_str(self):
+        """
+        Return a string that identifies the value-mapped element.
+        """
+        # pylint: disable=no-else-return
+        if isinstance(self.element, CIMProperty):
+            return _format("property {0!A} in class {1!A} (in {2!A})",
+                           self.propname, self.classname, self.namespace)
+        elif isinstance(self.element, CIMMethod):
+            return _format("method {0!A} in class {1!A} (in {2!A})",
+                           self.methodname, self.classname, self.namespace)
+        assert isinstance(self.element, CIMParameter)
+        return _format("parameter {0!A} of method {1!A} in class {2!A} "
+                       "(in {3!A})",
+                       self.parametername, self.methodname, self.classname,
+                       self.namespace)
+
     def __repr__(self):
         """
         Return a representation of the :class:`~pywbem.ValueMapping` object
         with all attributes, that is suitable for debugging.
         """
-        return "%s(_conn=%r, _namespace=%r, " \
-               "_classname=%r, _propname=%r, _methodname=%r, " \
-               "_parametername=%r, _element_obj=%r, " \
-               "_single_dict=%r, _range_tuple_list=%r, _unclaimed=%r)" % \
-               (self.__class__.__name__, self._conn, self._namespace,
-                self._classname, self._propname, self._methodname,
-                self._parametername, self._element_obj,
-                self._single_dict, self._range_tuple_list, self._unclaimed)
+        return _format(
+            "ValueMapping("
+            "_conn={s._conn!A}, "
+            "_namespace={s._namespace!A}, "
+            "_classname={s._classname!A}, "
+            "_propname={s._propname!A}, "
+            "_methodname={s._methodname!A}, "
+            "_parametername={s._parametername!A}, "
+            "_element_obj={s._element_obj!A}, "
+            "_b2v_single_dict={s._b2v_single_dict!A}, "
+            "_b2v_range_tuple_list={s._b2v_range_tuple_list!A}, "
+            "_b2v_unclaimed={s._b2v_unclaimed!A}, "
+            "_v2b_dict={s._v2b_dict!A})",
+            s=self)
 
     @property
     def conn(self):
@@ -555,24 +709,112 @@ class ValueMapping(object):
         """
 
         if not isinstance(element_value, (six.integer_types, CIMInt)):
-            raise TypeError("Element value is not an integer type: %s" %
-                            type(element_value))
+            raise TypeError(
+                _format("The value for value-mapped {0} is not "
+                        "integer-typed, but has Python type: {1}",
+                        self._element_str(), type(element_value)))
 
         # try single value
         try:
-            return self._single_dict[element_value]
+            return self._b2v_single_dict[element_value]
         except KeyError:
             pass
 
         # try value ranges
-        for range_tuple in self._range_tuple_list:
+        for range_tuple in self._b2v_range_tuple_list:
             lo, hi, values_str = range_tuple
             if lo <= element_value <= hi:
                 return values_str
 
         # try catch-all '..'
-        if self._unclaimed is not None:
-            return self._unclaimed
+        if self._b2v_unclaimed is not None:
+            return self._b2v_unclaimed
 
-        raise ValueError("Element value outside of the set defined by "
-                         "ValueMap: %r" % element_value)
+        raise ValueError(
+            _format("The value for value-mapped {0} is outside of the set "
+                    "defined by its ValueMap qualifier: {1!A}",
+                    self._element_str(), element_value))
+
+    def tobinary(self, values_str):
+        """
+        Return the integer value or values for a `Values` string, based upon
+        this value mapping.
+
+        Any returned integer value is represented as the CIM type of the
+        element (e.g. :class:`~pywbem.Uint16`).
+
+        If the `Values` string corresponds to a single value, that single value
+        is returned as an integer.
+
+        If the `Values` string corresponds to a value range (e.g. "1.." or
+        "..2" or "3..4"), that value range is returned as a tuple with two items
+        that are the lowest and the highest value of the range. That is the
+        case also when the value range is open on the left side or right side.
+
+        If the `Values` string corresponds to the `unclaimed` indicator "..",
+        `None` is returned.
+
+        Parameters:
+
+          values_str (:term:`string`):
+            The `Values` string for the element value.
+
+        Returns:
+
+          :class:`~pywbem.CIMInt` or tuple of :class:`~pywbem.CIMInt` or `None`:
+            The element value or value range corresponding to the `Values`
+            string, or `None` for unclaimed.
+
+        Raises:
+
+          ValueError: `Values` string outside of the set defined by `Values`.
+          TypeError: `Values` string is not a string type.
+        """
+
+        if not isinstance(values_str, six.string_types):
+            raise TypeError(
+                _format("The values string for value-mapped {0} is not "
+                        "string-typed, but has Python type: {1}",
+                        self._element_str(), type(values_str)))
+
+        try:
+            return self._v2b_dict[values_str]
+        except KeyError:
+            raise ValueError(
+                _format("The values string for value-mapped {0} is outside "
+                        "of the set defined by its Values qualifier: {1!A}",
+                        self._element_str(), values_str))
+
+    def items(self):
+        """
+        Generator that iterates through the items of the value mapping. The
+        items are the array entries of the `Values` and `ValueMap` qualifiers,
+        and they are iterated in the order specified in the arrays.
+        If the `ValueMap` qualifier is not specified, the default of consecutive
+        integers starting at 0 is used as a default, consistent with
+        :term:`DSP0004`.
+
+        Each iterated item is a tuple of integer value(s) representing the
+        `ValueMap` array entry, and the corresponding `Values` string.
+        Any integer value in the iterated items is represented as the CIM type
+        of the element (e.g. :class:`~pywbem.Uint16`).
+
+        If the `Values` string corresponds to a single element value, the
+        first tuple item is that single integer value.
+
+        If the `Values` string corresponds to a value range (e.g. "1.." or
+        "..2" or "3..4"), that value range is returned as a tuple with two items
+        that are the lowest and the highest value of the range. That is the
+        case also when the value range is open on the left side or right side.
+
+        If the `Values` string corresponds to the `unclaimed` indicator "..",
+        the first tuple item is `None`.
+
+        Returns:
+
+          :term:`iterator` for tuple of integer value(s) and `Values` string.
+        """
+
+        for values_str in self._v2b_dict:
+            element_value = self._v2b_dict[values_str]
+            yield element_value, values_str
